@@ -26,7 +26,9 @@ MAX_DISTANCE = 20
 SEQUENCE_LEN = 50
 SEQ_PER_START = 2
 
-MIN_WORD_COUNT = 10
+MIN_WORD_COUNT = 5
+
+IGNORE_NODES = (Annotation, )
 
 def _build_identifier_graph(tree):
     ## solve member reference and method invocation
@@ -35,31 +37,32 @@ def _build_identifier_graph(tree):
     # print(normalized_idens)
     if not (MIN_IDENTIFIER <= len(normalized_idens) <= MAX_IDENTIFIER):
         return dict()
-    graph = dict()
-    paths = list()
+    leaf2ancestors = defaultdict(dict)
     for path, ast_node in tree:
+        if isinstance(ast_node, IGNORE_NODES):
+            continue
+        path = (ast_node, ) + tuple(n for n in reversed(path) if isinstance(n, Node))
+        path = path[:MAX_DISTANCE]
         for child in ast_node.children:
             if not (isinstance(child, str) and child in identifiers):
                 continue
-            path += (Delimiter.split_camel(child, to_lower=True).replace(" ", "_"), )
-            paths.append(path)
-    for path1, path2 in itertools.combinations(paths, 2):
-        iden1, iden2 = path1[-1], path2[-1]
-        if iden1 == iden2:
+            leaf = Delimiter.split_camel(child, to_lower=True).replace(" ", "_")
+            for dis, ancestor in enumerate(path, start=1):
+                if ancestor in leaf2ancestors[leaf] and leaf2ancestors[leaf][ancestor] < dis:
+                    continue
+                leaf2ancestors[leaf][ancestor] = dis
+            
+    graph = dict()
+    for leaf1, leaf2 in itertools.combinations(leaf2ancestors.keys(), 2):
+        ancestors1, ancestors2 = leaf2ancestors[leaf1], leaf2ancestors[leaf2]
+        common_ancestors = set(ancestors1.keys()) & set(ancestors2.keys())
+        if len(common_ancestors) == 0:
             continue
-        offset = max(max(len(path1), len(path2)) - MAX_DISTANCE, 0)
-        index = 0
-        for i, (ele1, ele2) in enumerate(zip(path1[offset:], path2[offset:])):
-            if ele1 is not ele2:
-                index = i
-                break
-        dis = len(path1[index:]) + len(path2[index:])
+        dis = min(ancestors1[common_anc] + ancestors2[common_anc] for common_anc in common_ancestors)
         if dis > MAX_DISTANCE:
             continue
-        if (iden1, iden2) in graph and dis >= graph[iden1, iden2]:
-            continue
-        graph[iden1, iden2] = dis
-        graph[iden2, iden1] = dis
+        graph[leaf1, leaf2] = dis
+        graph[leaf2, leaf1] = dis
     return graph
 
 def _random_walk(graph):
@@ -74,18 +77,18 @@ def _random_walk(graph):
         total_weight = sum(weights)
         probs = [w / total_weight if total_weight > 0 else 1. for w in weights]
         transition_table[u] = neighbours, probs
-    # print(transition_table)
 
-    iden_seqs = list()    
+    iden_seqs = list()   
+    seq_len = min(len(transition_table), SEQUENCE_LEN) 
     for identifier in transition_table:
         for _ in range(SEQ_PER_START):
             seq = list()
             hop = 0
             cur_identifier = identifier
-            while hop < SEQUENCE_LEN:
+            while hop < seq_len:
                 # seq.extend(slicer.slice(cur_identifier))
                 seq.append(cur_identifier)
-                neighbours, probs = transition_table[identifier]
+                neighbours, probs = transition_table[cur_identifier]
                 cur_identifier = np.random.choice(neighbours, 1, probs)[0]
                 hop += 1
             iden_seqs.append(seq)
@@ -111,19 +114,19 @@ def _extract_for_code(code, level="method"):
     for tree in subtrees:
         words = set()
         for seq in _extract_for_subtree(tree):
+            # print(seq)
             delimited_seq = []
             for iden in seq:
-                iden = Delimiter.split_camel(iden, to_lower=True).replace(" ", "_")
                 delimited_seq.append(iden)
                 for word in iden.split("_"):
-                    words.add(word)
+                    word2count[word] = word2count[word] + 1
             iden_seqs.append(delimited_seq)
         for word in words:
             word2count[word] = word2count[word] + 1
         
     return iden_seqs, word2count
 
-def generate_corpus(input_file, sequence_file, meta_file, buf_size=1000):
+def generate_corpus(input_file, sequence_file, meta_file, level="method", buf_size=1000):
     with Path(input_file).open("rb") as f:
         codes = pickle.load(f)
     # print("finish loading code.")
@@ -136,7 +139,7 @@ def generate_corpus(input_file, sequence_file, meta_file, buf_size=1000):
         num = 0
         for pid, cid, code in codes:
             num += 1
-            _seqs, _word2count = _extract_for_code(code)
+            _seqs, _word2count = _extract_for_code(code, level=level)
             seqs.extend(_seqs)
             for word, count in _word2count.items():
                 word2count[word] = word2count[word] + count 
@@ -174,9 +177,9 @@ def train(corpus_files, meta_files, emb_path, workers=32):
                 with Path(fname).open("r", encoding="utf-8") as f:
                     sentences = f.readlines()
                     for sent in sentences:
-                        yield re.split(r"\s_", sent.strip())
+                        yield re.split(r"[\s_]", sent.strip())
 
-    fasttext = FastText(sg=1, hs=1, vector_size=100, window=10, min_count=MIN_WORD_COUNT, workers=workers)
+    fasttext = FastText(sg=1, hs=1, vector_size=100, window=15, min_count=MIN_WORD_COUNT, workers=workers)
     fasttext.build_vocab_from_freq(word2count)
     fasttext.train(Corpus(corpus_files), total_examples=seq_num, epochs=5)
     fasttext.wv.save(emb_path)
