@@ -1,4 +1,4 @@
 from codetoolkit.delimiter import Delimiter
 
 if __name__ == "__main__":
-    print(Delimiter.split_camel_strict("md5sumtest"))
\ No newline at end of file
+    print(Delimiter.split_camel("paycheck"))
\ No newline at end of file
